#!/usr/bin/env python

"""
mod_python classes.
"""

import Generic
from mod_python.util import parse_qs, FieldStorage
from mod_python import apache
try:
    from mod_python import Cookie
except ImportError:
    # NOTE: Should provide an alternative implementation.
    Cookie = None

class Transaction(Generic.Transaction):

    """
    mod_python transaction interface.
    """

    def __init__(self, trans):

        "Initialise the transaction using the mod_python transaction 'trans'."

        self.trans = trans
        self.response_code = apache.OK

    # Request-related methods.

    def get_request_stream(self):

        """
        A framework-specific method which returns the request stream for
        the transaction.
        """

        return self.trans

    def get_request_method(self):

        """
        A framework-specific method which gets the request method.
        """

        return self.trans.method

    def get_headers(self):

        """
        A framework-specific method which returns all request headers as a
        dictionary-like object mapping header names to values.
        NOTE: If duplicate header names are permitted, then this interface will
        NOTE: need to change.
        """

        return self.trans.headers_in

    def get_header_values(self, key):

        """
        A framework-specific method which returns a list of all request header
        values associated with the given 'key'. Note that according to RFC 2616,
        'key' is treated as a case-insensitive string.
        """

        return self.convert_to_list(self.trans.headers_in.get(key))

    def get_content_type(self):

        """
        A framework-specific method which gets the content type specified on the
        request, along with the charset employed.
        """

        return self.parse_content_type(self.trans.content_type)

    def get_content_charsets(self):

        """
        Returns the character set preferences.
        """

        return self.parse_content_preferences(self.trans.headers_in.get("Accept-Charset"))

    def get_content_languages(self):

        """
        A framework-specific method which extracts language information from
        the transaction.
        """

        return self.parse_content_preferences(self.trans.headers_in.get("Accept-Language"))

    def get_path(self):

        """
        A framework-specific method which gets the entire path from the request.
        """

        return self.trans.uri

    def get_path_info(self):

        """
        A framework-specific method which gets the "path info" (the part of the
        URL after the resource name handling the current request) from the
        request.
        """

        return self.trans.path_info

    def get_query_string(self):

        """
        A framework-specific method which gets the query string from the path in
        the request.
        """

        return self.trans.args or ""

    # Higher level request-related methods.

    def get_fields_from_path(self):

        """
        A framework-specific method which extracts the form fields from the
        path specified in the transaction. The underlying framework may refuse
        to supply fields from the path if handling a POST transaction.

        Returns a dictionary mapping field names to lists of values (even if a
        single value is associated with any given field name).
        """

        return parse_qs(self.get_query_string(), 1) # keep_blank_values=1

    def get_fields_from_body(self):

        """
        A framework-specific method which extracts the form fields from the
        message body in the transaction.

        Returns a dictionary mapping field names to lists of values (even if a
        single value is associated with any given field name).

        The mod_python.util.FieldStorage class may augment the fields from the
        body with fields found in the path.
        """

        storage = FieldStorage(self.trans, keep_blank_values=1)

        # Traverse the storage, finding each field value.

        fields = {}
        for field in storage.list:
            if not fields.has_key(field.name):
                fields[field.name] = []
            fields[field.name].append(field.value)
        return fields

    def get_user(self):

        """
        A framework-specific method which extracts user information from the
        transaction.

        Returns a username as a string or None if no user is defined.
        """

        return self.trans.user

    def get_cookies(self):

        """
        A framework-specific method which obtains cookie information from the
        request.

        Returns a dictionary mapping cookie names to cookie objects.

        NOTE: No additional information is passed to the underlying API despite
        NOTE: support for enhanced cookies in mod_python.
        """

        return Cookie.get_cookies(self.trans)

    def get_cookie(self, cookie_name):

        """
        A framework-specific method which obtains cookie information from the
        request.

        Returns a cookie object for the given 'cookie_name' or None if no such
        cookie exists.
        """

        return self.get_cookies().get(cookie_name)

    # Response-related methods.

    def get_response_stream(self):

        """
        A framework-specific method which returns the response stream for
        the transaction.
        """

        return self.trans

    def get_response_code(self):

        """
        Get the response code associated with the transaction. If no response
        code is defined, None is returned.
        """

        return self.response_code

    def set_response_code(self, response_code):

        """
        Set the 'response_code' using a numeric constant defined in the HTTP
        specification.
        """

        self.response_code = response_code

    def set_header_value(self, header, value):

        """
        Set the HTTP 'header' with the given 'value'.
        """

        self.trans.headers_out[self.format_header_value(header)] = self.format_header_value(value)

    def set_content_type(self, content_type):

        """
        A framework-specific method which sets the 'content_type' for the
        response.
        """

        self.trans.content_type = self.format_content_type(content_type)

    def set_cookie(self, cookie):

        """
        A framework-specific method which stores the given 'cookie' object in
        the response.
        """

        if Cookie:
            Cookie.add_cookie(self.trans, cookie)
        else:
            # NOTE: Should raise an exception or provide an implementation.
            pass

    def set_cookie_value(self, name, value, path=None, expires=None):

        """
        A framework-specific method which stores a cookie with the given 'name'
        and 'value' in the response.

        The optional 'path' is a string which specifies the scope of the cookie,
        and the optional 'expires' parameter is a value compatible with the
        time.time function, and indicates the expiry date/time of the cookie.
        """

        # NOTE: We just hope that Cookie converts Unicode arguments to US-ASCII.

        if Cookie:
            cookie = Cookie.Cookie(name, value)
            if expires is not None:
                cookie.expires = expires
            if path is not None:
                cookie.path = path
            Cookie.add_cookie(self.trans, cookie)
        else:
            # NOTE: Should raise an exception or provide an implementation.
            pass

    def delete_cookie(self, cookie_name):

        """
        A framework-specific method which adds to the response a request that
        the cookie with the given 'cookie_name' be deleted/discarded by the
        client.
        """

        # Create a special cookie, given that we do not know whether the browser
        # has been sent the cookie or not.
        # NOTE: Magic discovered in Webware.

        if Cookie:
            cookie = Cookie.Cookie(cookie_name, "")
            cookie.path = "/"
            cookie.expires = 0
            cookie.max_age = 0
            Cookie.add_cookie(self.trans, cookie)
        else:
            # NOTE: Should raise an exception or provide an implementation.
            pass

# vim: tabstop=4 expandtab shiftwidth=4
